"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const element_1 = __importDefault(require("../../dom/element"));
const invalid_node_1 = __importDefault(require("../../dom/invalid-node"));
const omitted_element_1 = __importDefault(require("../../dom/omitted-element"));
const custom_rule_1 = __importDefault(require("../../rule/custom-rule"));
exports.default = custom_rule_1.default.create({
    name: 'parse-error',
    defaultLevel: 'error',
    defaultValue: null,
    defaultOptions: null,
    async verify(document, messages) {
        const reports = [];
        // const message = await messages(`Values allowed for {0} attributes are {$}`, '"role"');
        let hasBody = false;
        await document.walk(async (node) => {
            if (node instanceof element_1.default || node instanceof omitted_element_1.default) {
                if (node.nodeName.toLowerCase() === 'body') {
                    hasBody = true;
                }
            }
            if (node instanceof invalid_node_1.default) {
                if (hasBody && node.raw.indexOf('<body') === 0) {
                    reports.push({
                        severity: node.rule.severity,
                        message: '"body"要素はDOMツリー上に既に暗黙的に生成されています。',
                        line: node.line,
                        col: node.col,
                        raw: node.raw,
                    });
                }
                else {
                    reports.push({
                        severity: node.rule.severity,
                        message: 'パースできない不正なノードです。',
                        line: node.line,
                        col: node.col,
                        raw: node.raw,
                    });
                }
            }
        });
        return reports;
    },
});
